{ Nullable }

procedure Nullable<T>.Clear;
begin
  FHasValue := nil;
end;

class function Nullable<T>.ConcreteTypeInfo: PTypeInfo; static;
begin
    Result := System.TypeInfo(T);
end;

class function Nullable<T>.ConcreteTypeName: string; static;
var
  LTypeInfo: PTypeInfo;
begin
  LTypeInfo := ConcreteTypeInfo;
  if LTypeInfo <> nil then
    Result := LTypeInfo^.Name
  else
    Result := '';
end;

class function Nullable<T>.ConcreteTypeKind: TTypeKind; static;
var
  LTypeInfo: PTypeInfo;
begin
  LTypeInfo := ConcreteTypeInfo;
  if Assigned(LTypeInfo) then
    Result := LTypeInfo^.Kind
  else
    Result := tkUnknown;
end;

function Nullable<T>.GetValue: T;
begin
  if not HasValue then
    raise EInvalidOperation.Create(SNullableTypeHasNoValue);
  Result := FValue;
end;

function Nullable<T>.GetHasValue: Boolean;
begin
  Result := FHasValue <> nil;
end;

constructor Nullable<T>.Create(AValue: T);
begin
  FValue := AValue;
  FHasValue := TNullableMockInterfacedObject.Instance;
end;

function Nullable<T>.GetValueOrDefault: T;
begin
  if HasValue then
    Result := FValue
  else
    Result := Default(T);
end;

function Nullable<T>.GetValueOrDefault(Default: T): T;
begin
  if not HasValue then
    Result := Default
  else
    Result := FValue;
end;

class operator Nullable<T>.NotEqual(ALeft, ARight: Nullable<T>): Boolean;
begin
  Result := (not ((ALeft.HasValue) or (ARight.HasValue)))
            or (ALeft.Value <> ARight.Value);

end;

class operator Nullable<T>.Equal(ALeft, ARight: Nullable<T>): Boolean;
begin
  if (ALeft.HasValue) and (ARight.HasValue) then
    Result := ALeft.Value = ARight.Value
  else
    Result := False;
end;

class operator Nullable<T>.Implicit(Value: Nullable<T>): T;
begin
  Result := Value.Value;
end;

class operator Nullable<T>.Implicit(Value: T): Nullable<T>;
begin
  Result := Nullable<T>.Create(Value);
end;

class operator Nullable<T>.Implicit(Value: Pointer): Nullable<T>;
begin
  if Value = nil then
    Result.Clear
  else
    raise EInvalidOperation.Create(SCannotAssignPointerToNullable);
end;

class operator Nullable<T>.Explicit(Value: Nullable<T>): T;
begin
  Result := Value.Value;
end;

class operator Nullable<T>.Implicit(Value: Variant): Nullable<T>;
begin
  if (VarIsNull(Value)) or (VarIsEmpty(Value)) then
    Result.Clear
  else
    Result := Value;
end;
