unit NullableTypes.Boolean;

{$mode delphi}

interface

uses
  TypInfo,
  NullableTypes.Exception,
  NullableTypes.SupportTypes;

type
  { Nullable<T> }

  Nullable<T> = record
  strict private
     {$INCLUDE nullabletype.base.privatemembers.inc}
  public
     {$INCLUDE nullabletype.base.publicmembers.inc}

     class operator LogicalNot(A: Nullable<T>): Nullable<T>;
     class operator LogicalAnd(A: Nullable<T>; B: Boolean): Nullable<T>;
     class operator LogicalOr(A: Nullable<T>; B: Boolean): Nullable<T>;
     class operator LogicalXor(A: Nullable<T>; B: Boolean): Nullable<T>;
  end;

  { Boolean Types }

  NullableBoolean  = Nullable<Boolean>;
  NullableByteBool = Nullable<ByteBool>;
  NullableWordBool = Nullable<WordBool>;
  NullableLongBool = Nullable<LongBool>;


implementation

uses
  Variants;

{$INCLUDE nullabletype.base.implementation.inc}

class operator Nullable<T>.LogicalNot(A: Nullable<T>): Nullable<T>;
begin
  Result := not A.Value;
end;

class operator Nullable<T>.LogicalAnd(A: Nullable<T>; B: Boolean): Nullable<T>;
begin
  Result := A.Value and B;
end;

class operator Nullable<T>.LogicalOr(A: Nullable<T>; B: Boolean): Nullable<T>;
begin
  Result := A.Value or B;
end;

class operator Nullable<T>.LogicalXor(A: Nullable<T>; B: Boolean): Nullable<T>;
begin
  Result := A.Value xor B;
end;

end.

