unit NullableTypes.Float;

{$mode delphi}

interface

uses
  TypInfo,
  NullableTypes.Exception,
  NullableTypes.SupportTypes;

type
  Nullable<T> = record
  strict private
     {$INCLUDE nullabletype.base.privatemembers.inc}
  public
     {$INCLUDE nullabletype.base.publicmembers.inc}
     {$INCLUDE nullabletype.comparisionoperators.interface.inc}
     {$INCLUDE nullabletype.unaryarithmeticoperators.interface.inc}

     class operator Add(A,B: Nullable<T>): Nullable<T>;
     class operator Subtract(A,B: Nullable<T>): Nullable<T>;
     class operator Multiply(A,B: Nullable<T>): Nullable<T>;
     class operator IntDivide(A: Nullable<T>; B: T): Nullable<T>;
     class operator Modulus(A: Nullable<T>; B: T): Nullable<T>;
  end;

  { Float Numeric Types }

  NullableReal     = Nullable<Real>;
  NullableSingle   = Nullable<Single>;
  NullableDouble   = Nullable<Double>;
  NullableExtended = Nullable<Extended>;
  NullableComp     = Nullable<Comp>;
  NullableCurrency = Nullable<Currency>;

implementation

uses
  Variants;

{ Nullable<T> }

{$INCLUDE nullabletype.base.implementation.inc}
{$INCLUDE nullabletype.comparisionoperators.implementation.inc}
{$INCLUDE nullabletype.unaryarithmeticoperators.implementation.inc}

class operator Nullable<T>.Add(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value + B.Value;
end;

class operator Nullable<T>.Subtract(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value - B.Value;
end;

class operator Nullable<T>.Multiply(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value * B.Value;
end;

class operator Nullable<T>.IntDivide(A: Nullable<T>; B: T): Nullable<T>;
begin
  Result := A div B;
end;

class operator Nullable<T>.Modulus(A: Nullable<T>; B: T): Nullable<T>;
begin
  Result := A mod B;
end;

end.

