unit NullableTypes.Ordinal;

{$mode delphi}

interface

uses
  TypInfo,
  NullableTypes.Exception,
  NullableTypes.SupportTypes;

type

  { Nullable<T> }

  Nullable<T> = record
  strict private
     {$INCLUDE nullabletype.base.privatemembers.inc}
  public
     {$INCLUDE nullabletype.base.publicmembers.inc}
     {$INCLUDE nullabletype.bitwiseoperators.interface.inc}
     {$INCLUDE nullabletype.comparisionoperators.interface.inc}
     {$INCLUDE nullabletype.unaryarithmeticoperators.interface.inc}

     class operator Add(A,B: Nullable<T>): Nullable<T>;
     class operator Subtract(A,B: Nullable<T>): Nullable<T>;
     class operator Multiply(A,B: Nullable<T>): Nullable<T>;
     class operator IntDivide(A,B: Nullable<T>): Nullable<T>;
     class operator Modulus(A,B: Nullable<T>): Nullable<T>;

     class operator Inc(A: Nullable<T>) : Nullable<T>;
     class operator Dec(A: Nullable<T>): Nullable<T>;
  end;


  { Ordinal Numeric Types }
  NullableByte       = Nullable<Byte>;
  NullableWord       = Nullable<Word>;
  NullableCardinal   = Nullable<Cardinal>;
  NullableLongword   = Nullable<Longword>;
  NullableQWord      = Nullable<QWord>;

  NullableShortInt   = Nullable<ShortInt>;
  NullableSmallInt   = Nullable<SmallInt>;
  NullableInteger    = Nullable<Integer>;
  NullableLongInt    = Nullable<LongInt>;
  NullableInt64      = Nullable<Int64>;

  NullableNativeInt  = Nullable<NativeInt>;
  NullableNativeUInt = Nullable<NativeUInt>;


implementation

uses
  Variants;

{ Nullable<T> }

{$INCLUDE nullabletype.base.implementation.inc}
{$INCLUDE nullabletype.bitwiseoperators.implementation.inc}
{$INCLUDE nullabletype.comparisionoperators.implementation.inc}
{$INCLUDE nullabletype.unaryarithmeticoperators.implementation.inc}

class operator Nullable<T>.Add(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value + B.Value;
end;

class operator Nullable<T>.Subtract(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value - B.Value;
end;

class operator Nullable<T>.Multiply(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value * B.Value;
end;

class operator Nullable<T>.IntDivide(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value div B.Value;
end;

class operator Nullable<T>.Modulus(A,B: Nullable<T>): Nullable<T>;
begin
  Result := A.Value mod B.Value;
end;

class operator Nullable<T>.Inc(A: Nullable<T>) : Nullable<T>;
begin
  Result := A.Value + 1;
end;

class operator Nullable<T>.Dec(A: Nullable<T>): Nullable<T>;
begin
  Result := A.Value - 1;
end;

end.

